@@ -22,11 +22,11 @@
   #include <omp.h>
 #else
 
-constexpr unsigned int omp_get_max_threads() {
+constexpr int omp_get_max_threads() {
   return 1;
 }
 
-constexpr unsigned int omp_get_thread_limit() {
+constexpr int omp_get_thread_limit() {
   return 1;
 }
 #endif
@@ -35,6 +35,7 @@ unsigned int OmpThread::nThread = OmpThread::nThreadDefault;
 
 const unsigned int OmpThread::maxThreads = 1024; // Cribbed from above.
 
+
 void OmpThread::init(unsigned int nThread_) {
   unsigned int ompMax = std::min(omp_get_max_threads(), omp_get_thread_limit());
 
